@@ -657,6 +657,47 @@ class Tool:
         if not os.path.exists(path):
             os.makedirs(path)
 
+    def dup_insert(self, table: str, datas: dict or list) -> list:
+        """
+        无则插入, 有则更新
+        :param table: 表名
+        :param datas: 数据
+        :return:
+        """
+
+        def analysis_dict(self, item, type="install"):
+            if type == "install":
+                key = item.keys()
+                value = item.values()
+                return ", ".join(key), ", ".join(["\"{}\"".format(pymysql.escape_string(v) if isinstance(v, str) else v) for v in value])
+            elif type == "update":
+                str_list = ["{}=\"{}\"".format(k, pymysql.escape_string(v) if isinstance(v, str) else v) for k, v in item.items()]
+                return ", ".join(str_list)
+
+        insert_url = """INSERT INTO {} ({}) VALUES ({}) ON DUPLICATE KEY UPDATE {}"""
+        if isinstance(datas, dict):
+            datas = [datas]
+        if isinstance(datas, list):
+            L.acquire()
+            try:
+                if not self.connection:
+                    self.connection = self.get_con()
+                cursor = self.connection.cursor()
+                for data in datas:
+                    key, value = analysis_dict(data, type="install")
+                    kv_up = analysis_dict(data, type="update")
+                    sql = insert_url.format(table, key, value, kv_up)
+                    try:
+                        cursor.execute(sql)
+                    except Exception as e:
+                        print(e, sql)
+                self.connection.commit()
+                cursor.close()
+            finally:
+                L.release()
+
+
+
 
 if __name__ == '__main__':
     tool = Tool('local')
